#include "contextswitch.h"

#if defined(_USE_SETJMP_LINUX_IA32_)


/*! \ingroup Gctxlinuxia32
\brief  Restaurer un contexte processeur. Le contexte courant est perdu.
\param ctx contexte  restaurer

C'est l'quivalent d'un \b goto sans espoir de reprise.
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est exactement un \c longjmp() de la libC
*/

void LinuxIA32FredContextRestore(LinuxIA32FredContext ctx)
{
	longjmp(ctx,1);
  /*  Il n'y a pas de retour possible en squence d'un tel appel  */
  KernelAbort(" FredContext Restore  : erreur fatale..... ");
} 

/*! \ingroup Gctxlinuxia32
\brief  Restaurer un contexte processeur. Le contexte courant est sauv.
\param  old contexte o sauvegarder
\param  ctx contexte  restaurer

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est exactement une combinaison de \c setjmp() pour la sauvegarde et \c longjmp() pour la restauration
*/

void LinuxIA32FredContextSwitch( LinuxIA32FredContext  old,LinuxIA32FredContext  ctx)
{ 
	if(setjmp(old))return; /* cas de reprise du contexte*/
	/* contexte courant sauvegard : on restaure un autre contexte*/
	longjmp(ctx,1);
	/* on ne revient jamais ici */
  KernelError(" FredContext switch  : erreur ????...???... ");
}
/*! \ingroup Gctxlinuxia32
\brief  Initialiser un contexte processeur. 
\param ctx contexte   initialiser
\param sp sommet de pile
\param entry adresse de la procdure  dmarrer

	On initialise un pointeur de pile et une adresse d'instruction.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est totalement dpendant du type de processeur et de l'implantation des fonction setjmp() et longjmp() car on doit connaitre les registres  modifier et leurs emplacements dans le jmp_buf.
*/

void  LinuxIA32FredContextInit( LinuxIA32FredContext  ctx, 
				char * sp , 
				void  (*entry)())
{
  ctx->__jmpbuf[JB_SP] = (int)sp;
	ctx->__jmpbuf[JB_PC] = (int)entry;
};
#endif
#if defined(_USE_SETJMP_SOLARIS_SPARC32_)

/*! \ingroup Gctxsolarissparc32
\brief  Restaurer un contexte processeur. Le contexte courant est perdu.
\param ctx contexte  restaurer

C'est l'quivalent d'un \b goto sans espoir de reprise.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est exactement un \c longjmp() de la libC
*/

void SolarisSparc32FredContextRestore(SolarisSparc32FredContext ctx)
{
	longjmp(ctx,1);
  /*  Il n'y a pas de retour possible en squence d'un tel appel  */
  KernelAbort(" FredContext Restore  : erreur fatale..... ");
} 
/*! \ingroup Gctxsolarissparc32
\brief  Restaurer un contexte processeur. Le contexte courant est sauv.
\param  old contexte o sauvegarder
\param  ctx contexte  restaurer

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est exactement une combinaison de \c setjmp() pour la sauvegarde et \c longjmp() pour la restauration
*/

void SolarisSparc32FredContextSwitch( SolarisSparc32FredContext  old,SolarisSparc32FredContext  ctx)
{ 
	if(setjmp(old))return; /* cas de reprise du contexte*/
	/* contexte courant sauvegard : on restaure un autre contexte*/
	longjmp(ctx,1);
	/* on ne revient jamais ici */
  KernelError(" FredContext switch  : erreur ????...???... ");
}
/*! \ingroup Gctxsolarissparc32
\brief  Initialiser un contexte processeur. 
\param ctx contexte   initialiser
\param sp sommet de pile
\param entry adresse de la procdure  dmarrer

	On initialise un pointeur de pile et une adresse d'instruction.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est totalement dpendant du type de processeur et de l'implantation des fonction setjmp() et longjmp() car on doit connaitre les registres  modifier et leurs emplacements dans le jmp_buf.
*/
void   SolarisSparc32FredContextInit( FredContext  ctx, 
				char * sp , 
				void  (*entry)() )
{
  ctx[ 1 ]= (long)sp;
	ctx[ 2 ]= (long)entry;
};

#endif
#if defined(_USE_ARCH_IA32_)
/*! \ingroup Gctxia32
\brief  Restaurer un contexte processeur. Le contexte courant est perdu.
\param ctx contexte  restaurer

C'est l'quivalent d'un \b goto sans espoir de reprise.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

C'est totalement dpendant du type de processeur et de la faon dont le code est gnr par le compilateur C (ici gcc 3.2) pour passer les paramtres dans la pile et entrer/sortir d'une procdure.  Une fois ceci connu (en deassemblant du code gnr par ex.), il suffit de dcider de l'ordre de sauvegarde des registres.

	Ici on dpile les registres de la pile dont le sommet est obtenu  partir du contexte.	

*/
 void IA32FredContextRestore(IA32FredContext ctx)
{
	/* esp pointeur sur le contexte ctx deplacement 4 relatif  esp*/
	asm( "movl 4(%esp), %esp");
	/* esp pointeur sur la pile  restaurer */
	asm( "movl 0(%esp), %esp");
	/*----- restaurer autres registres  ----- */ 
	asm( "popl %esi " );
	asm( "popl %edi " ); 
	asm( "popl %edx " ); 
	asm( "popl %ecx " ); 
	asm( "popl %ebx " ); 
	asm( "popl %eax " );
	asm( "popl %ebp " );
	asm( "ret" );
} 




/*! \ingroup Gctxia32
\brief  Restaurer un contexte processeur. Le contexte courant est sauv.
\param  old contexte o sauvegarder
\param  ctx contexte  restaurer

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est totalement dpendant du type de processeur et de la faon dont le code est gnr par le compilateur C (ici gcc 3.2) pour passer les paramtres dans la pile et entrer/sortir d'une procdure.  Une fois ceci connu (en deassemblant du code gnr par ex.), il suffit de dcider de l'ordre de sauvegarde des registres.

	Ici on empile les registres sur la pile, sauve le pointeur de pile dans le contexte old et on fait l'opration inverse depuis le contexte ctx.

*/
void IA32FredContextSwitch( IA32FredContext  old,IA32FredContext  ctx)
{ 
	/*  ebp et eax sauv sur  sur la pile */
	asm( "pushl %ebp  ");
	asm( "pushl %eax  ");
	/*ebp pointeur sur le contexte old  dplacement 12 relatif   esp */ 
	 asm(" movl 12(%esp), %ebp");
	/*eax pointeur sur le contexte new  dplacement 16 relatif   esp*/ 
	 asm(" movl 16(%esp), %eax");
	/*----- sauvver autres registres  ----- */ 
	asm( "pushl %ebx ");
	asm( "pushl %ecx ");   
	asm( "pushl %edx ");
	asm( "pushl %edi ");
	asm( "pushl %esi ");
	/* sauver esp dans le contexte old  */
	asm( "movl %esp ,0(%ebp)" );
	/* restaurer  nouvelle pile depuis  depuis eax (new) */
	asm( "movl 0(%eax),  %esp " );
	/*----- restaurer autres registres  ----- */ 
	asm( "popl %esi  ");
	asm( "popl  %edi "); 
	asm( "popl %edx  "); 
	asm( "popl  %ecx "); 
	asm( "popl  %ebx "); 
	asm( "popl  %eax ");
	asm( "popl  %ebp ");
	asm( "ret" );
}
/*! \ingroup Gctxia32
\brief  Initialiser un contexte processeur. 
\param ctx contexte   initialiser
\param sp sommet de pile
\param entry adresse de la procdure  dmarrer

	On initialise un registre pointeur de pile et un registre adresse d'instruction.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est totalement dpendant du type de processeur et de la faon dont le code est gnr par le compilateur C (ici gcc 3.2) pour passer les paramtres dans la pile et entrer/sortir d'une procdure.  Une fois ceci connu (en deassemblant du code gnr par ex.), il suffit de dcider de l'ordre de sauvegarde des registres.

	Ici, seuls l'initialisation du la sauvegarde des registres pointeur de pile et compteur ordinal est ncessaire.

*/
void   IA32FredContextInit( IA32FredContext  ctx, 
				char * sp , /* sommet de pile  */
				void  (*entry)() /* point de dmarrage */)
{
	/* sauve eax ebx et ecx  pour pouvoir les utiliser*/
	asm("pushl %eax;pushl %ebx; pushl %ecx"); 
  /* sauve esp  dans ecx */	
	asm(" movl %esp, %ecx");
 	/* faire pointer eax sur ctx (dplacement 16 relativement  ecx)*/
 	asm("  movl 16(%ecx), %eax");
 	/* esp pointe sur nouvelle pile sp  (deplacement 20 relativement  ecx)*/
 	asm("  movl 20(%ecx), %esp"); 
 	/* ebx pointe sur la procedure initiale entry (deplacement 24 relativement  ecx)*/
	asm("  movl 24(%ecx), %ebx");
	/* simulation sauvegarde registres    */
	asm(" pushl %ebx "); /* point de dmarrage */
	asm(" pushl $0 "); /* ebp bidon */
	asm(" pushl $0 "); /* eax  bidon */
	asm(" pushl $0 "); /* ebx  bidon */
 	asm(" pushl $0 "); /* ecx  bidon */
 	asm(" pushl $0 "); /* edx  bidon */
 	asm(" pushl $0 "); /* edi  bidon */
 	asm(" pushl $0 "); /* esi  bidon */
 	/* sauver pointeur de pile nouveau dans ctx */
 	asm(" movl %esp, 0(%eax)");
	/* restaure esp courant depuis ecx */
	asm(" movl %ecx ,%esp");
 	/* restaure  eax ebx et ecx */
	asm("popl %ecx;popl %ebx;popl %eax");
	/* retour */ 
 	asm(" ret");
}; 
#endif





