#include "interrupt.h"
#include <signal.h>




/*! \ingroup Git
\brief Dfinir un traitant d'interruption
\param sig numro de l'interruptio(signal UNIX)
\param handler procdure appele lors de l'interruption

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple carrossage de la procdure sigaction() d'UNIX.
	
*/
void FredSignalAttach(int sig,void(*handler)(int))
{ 
	struct sigaction siga;
	siga. sa_handler=handler;
  siga. sa_flags=0;
  sigemptyset(&(siga.sa_mask));
  sigaction(sig,&siga, (struct sigaction* )0);
}
/***************************************************************
*    Initialisation des traitants d'interruption                   
*
*  un traitant par dfaut est install pour tous les signaux  
*         sauf SIGKILL/SIGINT     
**************************************************************/

extern void defaultFredSignalHandler(int sig); /* forward reference */





/*! \ingroup Git
\brief Initialise les traitants par dfaut


\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	On initialise avec un traitant par dfaut sauf pour le signal SIGINT  pour conserver le comportement standard UNIX du ctrlC
*/


void FredSignalsInit()
{
	int i; int nsig;
	/* calcul du nombre de signaux */
  nsig=sizeof(sigset_t)*8;
  for(i=0;i<nsig;i++){ 
    if(i!=SIGINT){
			FredSignalAttach(i,defaultFredSignalHandler);
			/* on indique de redmarrer les appels systmes interrompus */
			siginterrupt(i,1); 
   }
  }
}

/*! \ingroup Git
\brief Traitant par dfaut des interruptions


\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	Simple impression d'un message d'erreur
*/
void defaultFredSignalHandler(int sig)
{ 
	printf("\nFRFED Default Signal Handler : signal %d \n ", sig);
}

