
#include "io.h"
#include "mask.h"
#include <stdarg.h>

/*! \ingroup Gio
\brief Imprime \b atomiquement sur la sortie standard
\param format format d'impression
\param ... suite variable d'argument
\return un code d'erreur

	Voir le manuel unix pour printf

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/

 int FredPrintf(const char *format, ...)
{
	int y;
	va_list ap;
  va_start(ap,format);
	FredMaskOn();
	y=vprintf(format,ap);
	FredMaskOff(); 
	return y;  
};

/*! \ingroup Gio
\brief Imprime \b atomiquement dans un fichier
\param stream le fichier cible
\param format format d'impression
\param ... suite variable d'argument
\return un code d'erreur

	Voir le manuel unix pour fprintf

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/


int FredFprintf(FILE *stream, const char *format, ...)
{  
	int y;va_list ap;
	FredMaskOn(); 
	y=vfprintf(stream,format,ap);
	FredMaskOff();
	return y;   
};

/*! \ingroup Gio
\brief Imprime \b atomiquement dans un buffer
\param str zone pour recevoir la chaine de caractre formate
\param format format d'impression
\param ... suite variable d'argument
\return un code d'erreur

	Voir le manuel unix pour sprintf

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/

int  FredSprintf(char *str, const char *format, ...)
{ 
	int y;va_list ap;
	FredMaskOn();
  y=vsprintf(str,format,ap);
	FredMaskOff();   
	return y;   
};


/*! \ingroup Gio
\brief lit et dcompose \b atomiquement un texte depuis un fichier source

\param format format d'analyse
\param ... suite variable d'argument
\return un code d'erreur

	Voir le manuel unix pour scanf
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/

 int FredScanf(const char *format, ...)
{
	int y;
	va_list ap;
  va_start(ap,format);
	FredMaskOn();
	y=vscanf(format,ap);
	FredMaskOff(); 
	return y;  
};

/*! \ingroup Gio
\brief lit et dcompose \b atomiquement un texte depuis l'entre standard
\param stream le fichier source
\param format format d'analyse
\param ... suite variable d'argument
\return un code d'erreur

	Voir le manuel unix pour fscanf

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/

int FredFscanf(FILE *stream, const char *format, ...)
{  
	int y;va_list ap;
	FredMaskOn(); 
	y=vfscanf(stream,format,ap);
	FredMaskOff();
	return y;   
};
/*! \ingroup Gio
\brief lit et dcompose \b atomiquement un texte depuis un buffer
\param str le buffer source
\param format format d'analyse
\param ... suite variable d'argument
\return un code d'erreur

	Voir le manuel unix pour sscanf

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/
int  FredSscanf(char *str, const char *format, ...)
{ 
	int y;va_list ap;
	FredMaskOn();
  y=vsscanf(str,format,ap);
	FredMaskOff();   
	return y;   
};




