/*! \file list.h
\brief Inclusion de la gestion de liste
*/
#ifndef _QUEUE_H
#define _QUEUE_H

/*! \ingroup Glist
\brief macro pour dfinir le pointeur vide

*/
#define NIL ((FredCell*)0)

/****************************************************************
****************************************************************
*                                                              
*  Gestion de files FIFO doublement chaines circulairement    
*                                                              

*****************************************************************
*****************************************************************/

/*! \ingroup Glist
\brief descripteur de cellule de liste 


Dans une liste circulaire  1 lment, celui ci est son propre successeur et prdcesseur. Il est donc doublement rechain sur lui-mme.
*/
typedef struct FredCell {  
	struct FredCell *next;/*!<  lment suivant de file*/
	struct FredCell * pred;/*!< lment pcdent de file*/
 }FredCell ; 

/*! \ingroup Glist
\brief macro pour dfinir une structure comme une cellule de liste

Pour dfinir une structure qyelconque comme pouvant tre insrer dans une liste, il suffit d'insrer FredCELL en premiere dfinition des composants de la structure 
\code
struct MyCell{ 
	FredCELL;
	int x;
	int y;
	...
};
\endcode
*/


#define FredCELL   FredCell _cell

/*! \ingroup Glist
\brief descripteur  de liste 

Une liste circulaire pointe sur le dernier lment. Pour une liste vide, ce derrnier lment est nul.
*/

typedef struct FredList 
{  
	FredCell * last; /*!< dernier lment de file*/
}FredList ;   



/*! \ingroup Glist
\brief Test de file vide

*/
#define FredListEmpty(Q) (((Q)->last) == (FredCell*)0)


/******************************************************************
* Insertion en fin de file                                    
******************************************************************/

extern void FredListPutLast(FredList *q,FredCell *b);

/******************************************************************
* Retrait en tte de file          
******************************************************************/
extern FredCell * FredListGetFirst(FredList *q) ;

/******************************************************************
* init  file          
******************************************************************/
extern void FredListInit(FredList *q) ;

/******************************************************************
* Test  file   vide                                
******************************************************************/



#endif
