
#include "mem.h"
#include "mask.h"
#include <stdlib.h>

/*! \ingroup Gmem
\brief Alloue \b atomiquement un bloc d'octets
\param x nombre d'octets  allouer
\return un pointeur sur un bloc ou 0

Voir le manuel unix pour malloc

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/
void * FredAtomicAlloc(int x)
{  
	FredMaskOn();
	void * y=malloc(x);    
	FredMaskOff(); 
	return y;  
};
/*! \ingroup Gmem
\brief Alloue \b atomiquement un vecteur de blocs d'octets
\param x nombre d'octets  allouer  par bloc
\param y nombre de blocs
\return un pointeur sur un vecteur de blocs ou 0

Voir le manuel unix pour calloc


\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/

void * FredAtomicCalloc(int x,int y)
{  
	FredMaskOn(); 
	void * z=calloc(x,y);   
	FredMaskOff();
	return z;   
};

/*! \ingroup Gmem
\brief Libre \b atomiquement un bloc prcdemment allou
\param x bloc  librer 

Voir le manuel unix pour free

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
  Simple mise en exclusion mutuelle de la procdure quivalente de la lib C.
*/
void  FredAtomicFree(void * x)
{  
	FredMaskOn();free(x);FredMaskOff();   
};




