#include "sema.h"

/*! \ingroup Gsema
\brief Initialise le descripteur de smaphore
\param s adresse descripteur smaphore
\param i nombre initial de jetons

\warning la valeur initiale du smaphore doit tre positive ou nulle
*/

void FredSemaphoreInit(FredSemaphore *s, int     i)
{
	FredMutexOn();/* dbut d'exclusion mutuelle */
	/* le semaphore existe et la valeur initiale est >=0 */
	if ((s !=(FredSemaphore *) 0)&&(i>=0)){
		s->counter = i;
		FredQueueInit(&(s->queue));/* file vide */
	}
	FredMutexOff();/* fin d'exclusion mutuelle */
}

/*************************************************/
/*! \ingroup Gsema
\brief Prend un jeton
\param s adresse descripteur smaphore

S'il n'y a pas de jeton, le thread se bloque. Il sera rveill  son tour ( file d'attente FIFO) quand un jeton lui sera donn. 

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation
*/

void   FredSemaphoreP(FredSemaphore * s)
{
	FredMutexOn();/* dbut d'exclusion mutuelle */
	s->counter--;
	if(s->counter < 0){
		/* pas de jeton :  on suspend le thread actif   */
    FredThreadSuspend(&(s->queue))  ; 
	}              
  FredMutexOff();/* fin d'exclusion mutuelle */
}

/*************************************************/
/*! \ingroup Gsema
\brief Depose un jeton
\param s adresse descripteur smaphore

S'il y a des threads en attente d'un jeton, on rveille le + ancien.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

*/
void     FredSemaphoreV(FredSemaphore * s)
{
	FredMutexOn();/* dbut d'exclusion mutuelle */
	s->counter++;
	if(s->counter <=0){
		/* il y a des demandeurs en attente */
		FredThreadWakeUp(&(s->queue));
	}
	FredMutexOff();/* fin d'exclusion mutuelle */
}





