/*! \file  thread.h
\brief Inclusion de la gestion des threads
*/
#ifndef _THREAD_H
#define _THREAD_H
#include "contextswitch.h"
#include "list.h"


/*! \ingroup Gthread
\brief  Le type d'une procdure initiale de  thread 
*/

typedef void* (*FredRunnable)(void*);

/*! \ingroup Gthread
\brief  L'identit d'un thread est un pointeur sur son descripteur
*/
typedef struct FredThreadDesc* FredThread;

/*! \ingroup Gthread
\brief  Dfinition du type queue de threads

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias d'une liste.
*/
typedef FredList FredQueue;

/*! \ingroup Gthread
\brief  Dfinition du thread inexistant

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias du pointeur nul de cellule de liste
*/
#define FredTHREAD_NIL (FredThread)NIL


/*! \ingroup Gcond
\brief Retirer le premier thread d'une file d'attente
\param Q  FredQueue  o mettre le thread courant 
\return le premier thread de la file ou 0 si file vide

\warning Cette procdure n'est utilisable qu'au sein d'une section en exclusion mutuelle.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias du retrait de liste, les conversions de type en sus.
*/
#define FredQueueGetFirst(Q) (FredThread)FredListGetFirst((FredList*)Q)

/*! \ingroup Gcond
\brief Insrer un thread dja bloqu dans une file d'attente
\param Q FredQueue o mettre le thread bloqu 
\param  th le  thread bloqu  insrer en queue

\warning Cette procdure n'est utilisable qu'au sein d'une section en exclusion mutuelle.

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias de l'insertion de liste, les conversions de type en sus.
*/


#define FredQueuePutLast(Q,th) FredListPutLast((FredList*)Q,(FredCell*)th)


/*! \ingroup Gcond
\brief Tester une queue  vide de threads 
\param Q queue 
\return 1 si vide. 0 sinon

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias du test de liste vide, les conversions de type en sus.
*/
#define FredQueueEmpty(Q) FredListEmpty((FredList*)Q)


/*! \ingroup Gcond
\brief Initialiser une queue vide de threads 
\param Q queue vide

\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias de l'initialisation de liste , les conversions de type en sus.
*/


#define FredQueueInit(Q) FredListInit((FredList*)Q);

/*! \ingroup Gmutex
\brief  Dbut de section critique


\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias de l'inhibition des interruptions
*/

#define FredMutexOn() FredMaskOn()

/*! \ingroup Gmutex
\brief  Fin de section critique


\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\htmlonly <hr WIDTH="100%"> \endhtmlonly 
\b Implantation

	C'est un simple alias de l'autorisation des interruptions
*/

#define FredMutexOff() FredMaskOff()

/*************************************************************************
*  suspension du thread actif dans une file                     
*     
*  DOIT ETRE FAIT MASQUE
**************************************************************************/
extern void FredThreadSuspend( FredQueue* q);


/*************************************************************************
*  rveil du premier thread suspendu  dans une file                 
*     On le met dans la file des prts
*  DOIT ETRE FAIT MASQUE
**************************************************************************/
extern void FredThreadWakeUp( FredQueue* q);


/**************************************************************
* cration & initialisation & lancement d'un thread  
**************************************************************/
extern FredThread   FredThreadSpawn( 
								int StackSize,
								void * ( *function)(void*),
                void *arg,
								void ** resref);
/**************************************************************
* pour terminer le thread de facon forcee 
*****************************************************************/ 
extern void             FredThreadExit(void *);

/*************************************************************************
*  Le  thread actif passe la main  un thread prt                 
**************************************************************************/
extern void FredThreadYield( );
/**************************************************************
* pour recuperer le numero du thread actif 
****************************************************************/  
extern FredThread		FredThreadCurrent();
           

/**************************************************************
* Initialisation du noyau de gestion de Threads    
**************************************************************/
extern void FredInitialize();
/**************************************************************
* Terminaison du noyau de gestion de Threads    
**************************************************************/
extern void  FredWaitTerminate();
/**************************************************************
* Terminaison brutale du noyau de gestion de Threads    
**************************************************************/
extern void  FredAbort(char *);

#endif
