/*! \file ech.h
\brief Gestion de la date et d'un chancier
*/
#ifndef _ECH_H
#define _ECH_H

/*! \ingroup Gech
\brief Prototype d'un traitant d'chance
	
L'action appele en fin d'chance reoit l'chance en paramtre.
*/
typedef void (*PhilEventHandler) (struct PhilEvent *);


/*! \ingroup Gech
\brief Descripteur de cellule de ech
	On rutilise la cellule de liste trie par priorit. La priorit est la date.
	On rajoute le traitement de fin d'chance.

*/
typedef struct PhilEvent{  
	PhilCELL;/*!<   chainage de liste et date */
	PhilEventHandler action;	/*!<  action  chance */
 }PhilEvent ; 




/*! \ingroup Gech
\brief descripteur  d'chancier

C'est une liste 
*/

typedef PhilList PhilEventList;   


/*! \ingroup Gech
\brief macro pour dfinir une structure comme une chance

Pour dfinir qu'une structure quelconque peut tre une chance, il suffit d'insrer PhilEVENT en premiere dfinition des composants de la structure 
\code
struct MyEvent{ 
	PhilEVENT;
	int x;
	int y;
	...
};
\endcode
*/


#define PhilEVENT   PhilEvent _event


                                
/*! \ingroup Gech
\brief Insertion  d'un venement
\param q  chancier o insrer
\param b  chance  insrer


L'insertion place la nouvelle chance  la suite des chances de mme date (ou de date infrieue) et devant celles de dates suprieures. On doit ventuellement (si insertion en tte ) relancer le dcomptage avec une nouvelle valeur.
*/

extern void PhilEventListInsert(PhilEventList *q,PhilEvent *b);

/*! \ingroup Gech
\brief Obtenir les  chances chues
\param q  chancier  consulter
\param  past liste des chances chues  (vide  l'appel)


	On obtient la liste des chances <=  la date courante. Cette liste peut tre vide.  Cette fonction sert  vider l'chancier aprs une interruption de fin d'chance. On relance ventuellement le dcompteur.

*/

extern void PhilEventListPurge(PhilEventList *q, PhilEventList* past  );

/*! \ingroup Gech
\brief Retrait d'une chance quelconque
\param q  liste o  retirer
\param b  cellule  retirer


	Une chance est annule (par ex. un signal est arriv avant la fin du dlai d'une attente sur une condition). On doit la retirer de la liste. Si cette chance tait la plus proche, on peut  recalculer ventuellement la prochaine chance et relancer le dcompte.
*/

extern void PhilEventListRetract(PhilEventList *q,PhilEvent *b) ;


/*!\ingroup Gech
\brief  Date courante
\return la date en microseconde
	

*/

      
extern long int PhilEventListNow();





        
/*! \ingroup Gech
\brief Initialisation de la liste
\param q liste  initialiser
\param date date initiale en microsecondes

initialise l'chance et lance le calcul de la date 

*/
extern void PhilEventListInit(PhilEventList *q, long int date) ;



#endif
