/*! \file priolist.h
\brief inclusion de la gestion par priorit de liste 
*/
#ifndef _PRIOQUEUE_H
#define _PRIOQUEUE_H

/*! \ingroup Gprio
\brief macro pour dfinir le pointeur vide

*/
#define NIL ((philCell*)0)

/****************************************************************
****************************************************************
*                                                              
*  Gestion de files FIFO doublement chaines circulairement    
*                                                              

*****************************************************************
*****************************************************************/

/*! \ingroup Gprio
\brief descripteur de cellule de liste 

La priorit est un entier >=0. La priorit maximum est 0.
Dans une liste circulaire  1 lment, celui ci est son propre successeur et prdcesseur. l est donc doublement rechain sur lui-mme. 
*/
typedef struct PhilCell {  
	struct PhilCell *next;/*!<  lment suivant de file*/
	struct PhilCell * pred;/*!< lment pcdent de file*/
	long int prio; /*!< priorit de l'lment */
 }PhilCell ; 

/*! \ingroup Gprio
\brief macro pour dfinir une structure comme une cellule de liste

Pour dfinir qu'une structure quelconque peyt tre insrer dans une liste, il suffit d'insrer PhilCELL en premiere dfinition des composants de la structure 
\code
struct MyCell{ 
	PhilCELL;
	int x;
	int y;
	...
};
\endcode
*/


#define PhilCELL   PhilCell _cell

/*! \ingroup Gprio
\brief descripteur  de liste 

Une liste circulaire pointe sur le dernier lment. Pour une liste vide, ce derrnier lment est nul.
*/

typedef struct PhilList 
{  
	PhilCell * last; /*!< dernier lment de file*/
}PhilList ;   



/*! \ingroup Gprio
\brief Test de file vide

*/


#define PhilListEmpty(Q) (((Q)->last) == (PhilCell*)0)


                                
/*! \ingroup Gprio
\brief Insertion en queue d'une cellule 



L'insertion en queue place la nouvelle cellule  la suite des cellules de mme priorit (ou de priorit suprieure) et devant les cellules de priorit infrieure.
*/

extern void PhilListPutLast(PhilList *q,PhilCell *b);

/*! \ingroup Gprio
\brief Retrait de la cellule de tte
\param q  liste o  retirer
\result  cellule retirer


	On retire toujours en tte o se trouve la cellule la + prioritaire. 
*/

extern PhilCell * PhilListGetFirst(PhilList *q) ;


        
/*! \ingroup Gprio
\brief Initialisation de la liste
\param q liste  initialiser
*/
extern void PhilListInit(PhilList *q) ;



#endif
